Module:    Scheduler
Synopsis:  A soft realtime scheduler
Author:    Ron Franke-Polz
Copyright: (C) 2000, Ron Franke-Polz.  All rights reserved.
License:   This software is provided 'as-is', without any express or implied warranty. 
           In no event will the author be held liable for any damages arising from 
           the use of this software.
           Permission is granted to anyone to use this software for any purpose, 
           including commercial applications, and to alter it and redistribute it 
           freely, subject to the following restrictions:
           1. The origin of this software must not be misrepresented; you must not 
              claim that you wrote the original software. If you use this software in 
              a product, an acknowledgment in the product documentation would be appreciated 
              but is not required.
           2. Altered source versions must be plainly marked as such, and must not 
              be misrepresented as being the original software.
           3. This notice may not be removed or altered from any source distribution.

// =====================================================================
// <scheduler-browser> Command table stuff
// =====================================================================

/// "File" command table

define command-table *file-command-table* (*global-command-table*)
  menu-item "Exit"       = exit-frame,
    documentation: "Exit the application";
end command-table *file-command-table*;


/// "Help" command table

define command-table *help-command-table* (*global-command-table*)
  menu-item "Topics..." = <help-on-topics>,
    documentation: "Show documentation for a topic";
  separator;
  menu-item "About..."  = <help-on-version>,
    documentation: "Show the About box for this application";
end command-table *help-command-table*;


/// Application command table

define command-table *scheduler-inspector-command-table* (*global-command-table*)
  menu-item "File" = *file-command-table*;
  menu-item "Help" = *help-command-table*;
end command-table *scheduler-inspector-command-table*;



// =====================================================================
// <scheduler-browser>
// =====================================================================

define constant $sleep-time :: <real> = 1.0;    // sleep 1 second

define frame <scheduler-browser> (<simple-frame>)

  slot logic-thread :: false-or(<thread>) = #f;

  slot task-queue :: <task-queue-mgr>,
    init-keyword: scheduler:;

  slot task-queue-entry :: false-or(<task-queue-entry>) = #f,
    init-keyword: entry:;

  // ---------------------------------------------------------------------
  // Task tree & details
  // ---------------------------------------------------------------------
  pane tree-pane (frame)
     make(<tree-control>, label: "Queues",
                          label-key: name,
                          roots: frame.task-queue.children,
                          activate-callback: method (x)
                                               // focus on this <task-queue-entry>
                                               frame.task-queue-entry := x.gadget-value;

                                               // extract it's info
                                               frame.task-entry-name.gadget-value := frame.task-queue-entry.name;
                                               frame.enabled-indicator.gadget-value := frame.task-queue-entry.enabled?;
                                               frame.threaded-indicator.gadget-value := frame.task-queue-entry.threaded?;
                                             end,
                          children-predicate: children?,
                          children-generator:
                            method (x)
                              x.children;
                            end);

  pane task-entry-name (frame)
    make(<text-field>, text: "??");

  pane enabled-indicator (frame)
    make(<check-button>, label: "enabled?");

  pane threaded-indicator (frame)
    make(<check-button>, label: "threaded?");

  pane set-button (frame)
    make(<button>, label: "set",
                   activate-callback: method (x)
                                        if (frame.task-queue-entry)
                                          frame.task-queue-entry.enabled? := frame.enabled-indicator.gadget-value;
                                          frame.task-queue-entry.threaded? := frame.threaded-indicator.gadget-value;
                                        end if;
                                      end);

  pane task-details (frame)
    make(<group-box>, child: vertically (spacing: 4)
                               frame.task-entry-name;
                               frame.enabled-indicator;
                               frame.threaded-indicator;
                               frame.set-button;
                             end,
                      label: "task entry");


  pane main-pane (frame)
    horizontally (spacing: 4)
      frame.tree-pane;
      frame.task-details;
    end;

  layout (frame)
    frame.main-pane;

  command-table (frame)
    *scheduler-inspector-command-table*;
  //*** MAYBE INCLUDE A TOOL BAR ***
  /* tool-bar (frame)
       make-command-tool-bar(frame-manager(frame), frame); */

  status-bar (frame)
    make(<status-bar>);

  keyword title: = $application-name;

end frame <scheduler-browser>;



// ---------------------------------------------------------------------
// initialize
// ---------------------------------------------------------------------
define method initialize (frame :: <scheduler-browser>, #keys)
    next-method(frame);
    start-logic-loop(frame);
end method;

// ---------------------------------------------------------------------
// update-gadgets
// ---------------------------------------------------------------------
define method update-gadgets (frame :: <scheduler-browser>)

    while (#t)
      // nothing to do yet ...
      sleep ($sleep-time);
    end while;

end method;


// ---------------------------------------------------------------------
// start-logic-loop
// ---------------------------------------------------------------------
define method start-logic-loop (frame :: <scheduler-browser>)
    frame.logic-thread := make(<thread>,
                               function: method ()
                                           update-gadgets(frame);
                                         end,
                               name: "Scheduler Browser logic loop");
end method;



/// Help commands

define method do-execute-command
    (frame :: <scheduler-browser>, command :: <help-on-topics>) => ()
  //*** REMOVE THIS IF YOU ADD A HELP SOURCE ***
  notify-user("No help available.", owner: frame)
end method do-execute-command;

define method do-execute-command
    (frame :: <scheduler-browser>, command :: <help-on-version>) => ()
  //*** PUT UP A NICE ABOUT BOX ***
  notify-user(application-full-name(), owner: frame)
end method do-execute-command;


/// Start the scheduler-inspector

define method start-scheduler-inspector
    () => ()
  start-frame(make(<scheduler-browser>));
end method start-scheduler-inspector;


